@@ -1,5 +1,6 @@
 //! Runtime detection of whether we're in node.js or a browser.
 
+use alloc::string::String;
 use wasm_bindgen::prelude::*;
 
 #[wasm_bindgen]
@@ -12,6 +13,11 @@ extern "C" {
     #[wasm_bindgen(method, getter, structural)]
     fn constructor(me: &Scope) -> Constructor;
 
+    #[wasm_bindgen(method, getter, structural, js_name = Deno)]
+    fn deno(me: &Scope) -> Option<Deno>;
+
+    type Deno;
+
     type Constructor;
     #[wasm_bindgen(method, getter, structural)]
     fn name(me: &Constructor) -> String;
@@ -24,8 +30,13 @@ pub fn detect() -> Runtime {
     // only be true in browsers.
     match js_sys::global().unchecked_into::<This>().self_() {
         Some(scope) => match scope.constructor().name().as_str() {
-            "DedicatedWorkerGlobalScope" | "SharedWorkerGlobalScope" => Runtime::Worker,
-            _ => Runtime::Browser,
+            "DedicatedWorkerGlobalScope"
+            | "SharedWorkerGlobalScope"
+            | "ServiceWorkerGlobalScope" => Runtime::Worker,
+            _ => match scope.deno() {
+                Some(_) => Runtime::Node,
+                _ => Runtime::Browser,
+            },
         },
         None => Runtime::Node,
     }
